/*
 * Part of the GaBIEn project.
 * (Graphics and Basic Input Engine)
 *
 *        DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *                    Version 2, December 2004 
 *
 * Modified by gamemanj <asdd2808@gmail.com> to remove expletives
 * Original copyright (C) 2004 Sam Hocevar <sam@hocevar.net> 
 *
 * Everyone is permitted to copy and distribute verbatim or modified 
 * copies of this license document, and changing it is allowed as long 
 * as the name is changed. 
 *
 *            DO WHATEVER YOU WANT TO PUBLIC LICENSE 
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION 
 *
 *  0. You just DO WHATEVER YOU WANT TO.
 */
package gabien.music;

import gabien.ISoundDriver;

public class PiyoPiyoPlayingNote {
    public double secondsToPositionRatio = 1;
    public double position = 0;
    public double[] compiledEnvelopeL = new double[64];
    public double[] compiledEnvelopeR = new double[64];
    public ISoundDriver.IChannel channel;

    public PiyoPiyoPlayingNote(ISoundDriver.IChannel channel,
            PiyoPiyoWaveTrack wTrack, int pos, int pan) {
        this.channel = channel;
        // Calculate envelope with pan to simplify code later
        double lValue = 1;
        double rValue = 1;
        lValue *= doPan(wTrack.volume, pan, true, false);
        rValue *= doPan(wTrack.volume, pan, false, false);
        for (int i = 0; i < 64; i++) {
            double normalValue = wTrack.envelope[i] / 127.0d;
            compiledEnvelopeL[i] = normalValue * lValue;
            compiledEnvelopeR[i] = normalValue * rValue;
        }
        // PlaySound only accepts this format.
        short[] compiledSound = new short[256];
        for (int i = 0; i < 256; i++)
            compiledSound[i] = (short) (wTrack.waveForm[i] << 8);
        pos += wTrack.octave * 12;// Calculated by playing around in PiyoPiyo :)

        // Thank you liborganya, as it's where I got this from.
        double freq = 8363d * Math.pow(2.0d, pos / 12.0d) / 22050d;

        // 0.03 seconds at pos 1, and it seems to go up by the 0.005 range.
        channel.playSound(freq, compiledEnvelopeL[0], compiledEnvelopeR[0],
                compiledSound, true);
        secondsToPositionRatio = 22050.0d / wTrack.length;
    }

    protected static double doPan(double volume, int pan, boolean b,
            boolean drums) {
        double volumeValue = volume / 300.0d;
        volumeValue += drums?1.0d:0.25d;
        double panMul = pan * (1.0d / 6);
        if (b)
            panMul = 1.0d - panMul;
        return volumeValue * panMul;
    }

    public boolean updateEnvelope(double deltaTime) {
        position += deltaTime * secondsToPositionRatio;
        int tPosition = (int) (position * 64);
        if (tPosition < 0)
            return false;
        if (tPosition >= 64) {
            channel.setVolume(0, 0);
            return true;
        }
        channel.setVolume(compiledEnvelopeL[tPosition],
                compiledEnvelopeR[tPosition]);
        return false;
    }
}
